/*
 * @LoQuake.java Version 1.1 01/30/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

import java.net.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import cmn.cmnStruct;
import iqstrat.iqstratStruct;

import lith.lithology.io.ReadLithologySymbolsXMLFile;

import quake.gui.quakeStartAppPanel;

/** Class LoQuake
 *  <p> This Class is the starting point for the LoQuake Web App.
 *
 *  @version 1.1 01/30/2015
 *  @author  John R. Victorine
 */

public class LoQuake extends Applet
{
  // Runtime object

  private Runtime r = Runtime.getRuntime();

  // Web Application is runing Stand Alone Indicator

  private boolean isStandalone = false;

  private String  sKID            = new String("0"); // LAS KID
  private String  sKID_Standalone = new String("0");

  // LAS Viewer Global Variables

  private iqstratStruct stStruct = new iqstratStruct();
  private cmnStruct     stCMN    = new cmnStruct();
  private String        sPath    = new String(".");


  /** getParameter()
   *  <p> This method wil get a parameter value
   *  @param key = The Name of the Parameter
   *  @param def = The Object Type of the Parameter
   *  @return Parameter
   */

  public String getParameter(String key, String def)
  {
    return (isStandalone ? System.getProperty(key, def) :
            (getParameter(key) != null ? getParameter(key) : def));
  }

  /** Constructor LoQuake()
   *  <p> This is the Constructor for this class.
   */

  public LoQuake() {  }

  /** Method init()
   *  <p> This Method will retrieve the data values from the HTML or the main()
   *      arguments and run the Stratigraphic Data Collection and Biostratigraphy LoQuake
   *      Viewer Module.
   */

  public void init()
  {
    try
    {
      sKID = this.getParameter("sKID", sKID_Standalone);
      if (sKID.equals("undefined"))
        sKID = new String("0");
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will launch the Field LAS Catalog.
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
    quakeStartAppPanel panel = null;

    // initialize Global LAS Data Structure

    stCMN.bStandalone   = isStandalone;

    // Retrieve KGS Image Icon for printing

    if (!isStandalone)
    {
      stCMN.urlDirCodeBase = getCodeBase();
      stCMN.appletContext  = getAppletContext();
      stCMN.pIcon          = getImage(getCodeBase(), cmn.cmnStruct.ICON);
      stCMN.pIcon2         = getImage(getCodeBase(), cmn.cmnStruct.ICON2);
    }
    else
    {
      URL url = null;

      stCMN.urlDirCodeBase =  getClass().getClassLoader().getResource(".");

      ClassLoader cl = this.getClass().getClassLoader();
      sPath = cl.getResource("images").toString();
      stCMN.sImagePath = new String(sPath);

      try
      {
        url = new URL(sPath+"\\kgsSm.png");
        stCMN.pIcon = Toolkit.getDefaultToolkit().getImage(url);
        url = new URL(sPath+"\\kgsSm2.png");
        stCMN.pIcon2 = Toolkit.getDefaultToolkit().getImage(url);
      }
      catch (java.net.MalformedURLException e)
      {
        e.printStackTrace();
        String sMessage = new String("LoQuake(): \n" +
                              e.getMessage() + "\n" +
                              "Couldn't create image: " + sPath+"\\kgsSm.gif");
        JOptionPane.showMessageDialog((Component) null,
                                      sMessage,
                                      "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
      }
    }

    // Get the Path of the XML Files stored in JAR File.

    ClassLoader cl   = this.getClass().getClassLoader();
    sPath            = cl.getResource("images").toString();
    stCMN.sImagePath = new String(sPath);
    stCMN.cl         = cl;

    stStruct.stCMN = stCMN;

    // Read in Lookup Data

    readLithologySymbols();  // Read the Lithology Symbols Data

    // Display the Panel

    if (!isStandalone)
    {
      panel = new quakeStartAppPanel( null, stStruct );

      this.setLayout(new BorderLayout());
      this.add(panel, BorderLayout.CENTER);
    }
    else
    {
      JFrame frame = new JFrame();

      frame.setTitle("SELECT");
      panel = new quakeStartAppPanel( frame, stStruct );
      frame.getContentPane().add(panel, null);

      // Set the Size and Display the Frame

      frame.setSize(300, 300);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      frame.setLocation((d.width - frame.getSize().width)/2,
                        (d.height - frame.getSize().height)/2);

      frame.setResizable(false);
      frame.setVisible(true);
    }
  }

  /* ======================================================================= *
   * ------------------------ READ DATA XML METHODS ------------------------ *
   * ======================================================================= */

  /** Method readLithologySymbols()
   * <p> This method will read in the Lithology Plot Symbols
   */

  private void readLithologySymbols()
  {
    ReadLithologySymbolsXMLFile pData =
        new ReadLithologySymbolsXMLFile(
          lith.lithology.io.ReadLithologySymbolsXMLFile.URL);

    stStruct.stSymbols = pData.Process( cmn.cmnStruct.LITHOLOGY_XML );
//lith.lithology.lithologySymbolsUtility.print(stStruct.stSymbols);
  }

  /* ======================================================================= *
   * --------------------------- APPLET METHODS ---------------------------- *
   * ======================================================================= */

  /** Method start()
   *  <p> This method will start the applet
   */

  public void start() { }

  /** Method stop()
   *  <p> This method will stop the applet
   */

  public void stop() { }

  /** Method destroy()
   *  <p> This method will destroy the applet
   */

  public void destroy() { }

  /** Method getAppletInfo()
   *  <p> This method will return Applet information
   *  @return - Biostratigraphy LoQuake Plot
   */

  public String getAppletInfo() { return ("LAS LoQuake Applet"); }

  /** Method getParameterInfo()
   *  <p> This method will get the parameter info
   *  @return pinfo - Array of Parameters Expected.
   */

  public String[][] getParameterInfo()
  {
    String[][] pinfo = { {"sKID",  "String", "User Primary Key"}, };

    return pinfo;
  }

  /** Method main()
   *  <p> This method will allow this Module to run either as an Application or
   *      an Applet.
   *  @param args = The Arguments passed in to query for the production data
   */

  public static void main(String[] args)
  {
    LoQuake applet = new LoQuake();

    applet.isStandalone = true;
    applet.sKID_Standalone = new String("0");
    if (args.length > 0)
      applet.sKID_Standalone = new String(args[0]);

    applet.init();
    applet.start();
  }
}


/*
 *  @version 1.1 01/30/2015
 *  @author  John Victorine
 */
