/*
 * @brineStandardTools.java Version 1.1 06/01/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine;

/** Class brineStandardTools
 *  <p> This Class holds the standard tool information to map the brine
 *      curves to standard names for plotting.
 *
 *  List composed from,
 *  Common Cations, Anions, Acids, Salts and Hydrate Nomenclature
 *  Missouri University of Science and Technology
 *  http://web.mst.edu/~tbone/subjects/tbone/chem1/Chem%201%20Compound%20Sheet%20FS08.pdf
 */

public class brineStandardTools
{
  public static final int _KEY    = 0;  // KEY

  // AGE
  public static final int _FORM   = 1;  // Primary Formation
  public static final int _AGE    = 2;  // Age of Formation

  // Depth
  public static final int _TOP    = 3;  // Top Depth
  public static final int _BASE   = 4;  // Base Depth

  // Other Measured Well Data
  public static final int _SPGR   = 5;  // Specific Gravity
  public static final int _SIGMA  = 6;  // Specific Conductivity
  public static final int _PH     = 7;  // PH
  public static final int _DEG    = 8;  // Degree F
  public static final int _OHM    = 9;  // Resistivity (Rw)
  public static final int _OHM75  = 10; // Resistivity at 75 deg
  public static final int _OHME   = 11; // Estimated Rw

  // .. Alkali Metal
  public static final int _LI     = 12; // Lithium            1+
  public static final int _NA     = 13; // Sodium             1+
  public static final int _K      = 14; // Potassium          1+
  public static final int _RB     = 15; // Rubidium           1+
  public static final int _CS     = 16; // Cesium             1+
  public static final int _NA_K   = 17; // Sodium/Potasium    1+

  // ..  Alkali Earth Metal
  public static final int _BE     = 18; // Beryllium          2+
  public static final int _MG     = 19; // Magnesium          2+
  public static final int _CA     = 20; // Calcium            2+
  public static final int _SR     = 21; // Strontium          2+
  public static final int _BA     = 22; // Barium             2+

  // ..  VIB
  public static final int _CR_II  = 23; // Chromium (II)   chromous  2+
  public static final int _CR_III = 24; // Chromium (III)  chromic   3+

  // ..  VIIB
  public static final int _MN_II  = 25; // Manganese (II)  manganous 2+
  public static final int _MN_III = 26; // Manganese (III) manganic  3+

  // ..  VIIIB
  public static final int _FE_II  = 27; // Iron (II)    ferrous      2+
  public static final int _FE_III = 28; // Iron (III)   ferric		 3+
  public static final int _CO_II  = 29; // Cobalt (II)  cobaltous    2+
  public static final int _CO_III = 30; // Cobalt (III) cobaltic     3+
  public static final int _NI_II  = 31; // Nickel (II)  nickelous    2+
  public static final int _NI_III = 32; // Nickel (III) nickelic     3+

  // ..  IB
  public static final int _CU_I   = 33; // Copper (I)   cuprous      1+
  public static final int _CU_II  = 34; // Copper (II)  cupric       2+
  public static final int _AG     = 35; // Silver                    1+
  public static final int _AU     = 36; // Gold (I)     aurous       1+
  public static final int _AU_III = 37; // Gold (III)   auric        3+

    // ..  IIB
  public static final int _ZN     = 38; // Zinc                      2+
  public static final int _CD     = 39; // Cadmium                   2+
  public static final int _HG     = 40; // Mercury (I)  mercurous Hg2 2+
  public static final int _HG_II  = 41; // Mercury (II) mercuric     2+

  // ..  IIIA
  public static final int _AL     = 42; // Aluminium                 3+

  //.. VA
  public static final int _SB_III = 43; // Antimony (III)  antimonous  3+
  public static final int _SB_V   = 44; // Antimony (V)    antimonic   5+
  public static final int _BI_III = 45; // Bismuthous(III) bismuthous  3+
  public static final int _BI_V   = 46; // Bismuth(V)      bismuthic   5+

  // ..  IVA
  public static final int _SN_II  = 47; // Tin (II)  stannous          2+
  public static final int _SN_IV  = 48; // Tin (IV)  stannic           4+
  public static final int _PB_II  = 49; // Lead (II) plumbous          2+
  public static final int _PB_IV  = 50; // Lead (IV) plumbic           4+

  // .. Other
  public static final int _NH4    = 51; // Ammonium                    1+

  // Common Anions

  public static final int _F      = 52; // Flouride        1-
  public static final int _CL     = 53; // Chloride        1-
  public static final int _BR     = 54; // Bromide         1-
  public static final int _I      = 55; // Iodide          1-

  public static final int _OH     = 56; // Hydroxide       1-

  public static final int _BO3    = 57; // Borate
  public static final int _CO3    = 58; // Carbonate       2-
  public static final int _HCO3   = 59; // Bicarbonate     1-

  public static final int _CLO    = 60; // Hypochlorite    1-
  public static final int _CLO2   = 61; // Chlorite        1-
  public static final int _CLO3   = 62; // Chlorate        1-
  public static final int _CLO4   = 63; // Per chlorate    1-

  public static final int _CN     = 64; // Cyanide         1-
  public static final int _NCO    = 65; // Cyanate         1-
  public static final int _OCN    = 66; // Isocyanate      1-
  public static final int _SCN    = 67; // Thiocyanite     1-

  public static final int _N      = 68; // Nitride         3-
  public static final int _N3     = 69; // Azide           1-
  public static final int _NO2    = 70; // Nitrite         1-
  public static final int _NO3    = 71; // Nitrate         1-

  public static final int _CRO4   = 72; // Chromium (II)   Chromate    2-
  public static final int _CR2O7  = 73; // Chromium (III)  Dichormate  2-

  public static final int _MNO4   = 74; // Permanganate    1-

  public static final int _P      = 75; // Phosphide       3-
  public static final int _PO4    = 76; // Phosphate       3-
  public static final int _PO3    = 77;  // Phosphite       3-
  public static final int _HPO4   = 78; // Monohydrogen Phosphate 2-
  public static final int _H2PO4  = 79; // Dihydrogen Phosphate   1-

  public static final int _AS     = 80; // Arsinide        3-
  public static final int _SE     = 81; // Selenide        2-

  public static final int _S      = 82; // Sulfide            2-
  public static final int _HS     = 83; // Hydrogen Sulfide   1-
  public static final int _SO3    = 84; // Sulphite           2-
  public static final int _HSO3   = 85; // Hydrogen Sulphite  1-
  public static final int _S2O3   = 86; // Thiosulphate       2-
  public static final int _SO4    = 87; // Sulfate            2-
  public static final int _HSO4   = 88; // Bisulfate          1-

  public static final int _SOLID  = 89; // Total Solids
  public static final int _TDS    = 90; // Total Solids Computed

  // Well Header Information Data

  public static final int _NAME   = 91; // Well Name
  public static final int _API    = 92; // API-Number
  public static final int _FIELD  = 93; // Field
  public static final int _COUNTY = 94; // County
  public static final int _STATE  = 95; // State
  public static final int _LOC    = 96; // Location
  public static final int _LAT    = 97; // Latitude
  public static final int _LONG   = 98; // Longitude
  public static final int _KB     = 99; // Kelly Bushing
  public static final int _GL     = 100; // Ground Level
  public static final int _DF     = 101; // Derrick Floor
  public static final int _TGT    = 102; // TGT
  public static final int _SRC    = 103; // Source
  public static final int _DATE   = 104; // Date

  public static final int TOTAL   = 91; // TOTAL
  public static final int TOTAL_A = 105; // TOTAL With Well Header Data

  // Column Identifiers

  public static final int _C_KEY  = 0;
  public static final int _C_MNEM = 1;
  public static final int _C_DESC = 2;
  public static final int _C_UNIT = 3;
  public static final int _C_TYPE = 4;
  public static final int _C_VAL  = 5;
  public static final int _C_MOL  = 6;
  public static final int _C_SEL  = 7;

  public static final String CURVES[][] =
  {                                                                             // gm/mole
    // KEY          Mnemonic     Description            Units   TYPE     Valence  mMolar wt  *
    { "B_KEY",      "KEY",       "Fluid Primary Key",   "NONE", "KEY",       "",    "",     ""  },
    { "B_FORM",     "FORM",      "Formation",           "NONE", "Formation", "",    "",     ""  },
    { "B_AGE",      "AGE",       "Formation Age",       "NONE", "AGE",       "",    "",     ""  },
    // Depth Range of Sample
    { "B_TOP",      "TOP",       "Depth Top",           "ft",   "DEPTH",     "",    "",     ""  },
    { "B_BASE",     "BASE",      "Depth Base",          "ft",   "DEPTH",     "",    "",     ""  },

    // Other Measured Well Data
    { "B_SPGR",     "SPGR",      "Specific Gravity",      "NONE", "DATA",    "",    "",     ""  },
    { "B_SIGMA",    "SIGMA",     "Specific Conductivity", "mho",  "COND",    "",    "",     ""  },
    { "B_PH",       "PH",        "PH",                    "NONE", "PH",      "",    "",     ""  },
    { "B_DEG",      "DEG",       "Temperature (F)",       "degrees F", "TEMP", "",  "",     ""  },
    { "B_OHM",      "OHM",       "Resistivity (Rw)",      "ohm",  "OHM",     "",    "",     ""  },
    { "B_OHM75",    "OHM75",     "Resistivity @ 75 deg",  "ohm",  "OHM",     "",    "",     ""  },
    { "B_OHME",     "OHME",      "Estimated Rw",          "ohm",  "OHM",     "",    "",     ""  },

    // Common Cations

    // .. Alkali Metal
    { "B_Li",       "Li",        "Lithium",               "mg/l", "COMP", "1", "6.94",     ""  },
    { "B_Na",       "Na",        "Sodium",                "mg/l", "COMP", "1", "22.99",    "*"  },
    { "B_K",        "K",         "Potassium",             "mg/l", "COMP", "1", "39.10",    "*"  },
    { "B_RB",       "Rb",        "Rubidium",              "mg/l", "COMP", "1", "85.468",   ""  },
    { "B_CS",       "Cs",        "Cesium",                "mg/l", "COMP", "1", "132.91",   ""  },
    { "B_NA_K",     "Na_K",      "Sodium Potassium",      "mg/l", "COMP", "1", "31.05",    ""  },
    // ..  Alkali Earth Metal
    { "B_Be",       "Be",        "Beryllium",             "mg/l", "COMP", "2", "9.01",     ""  },
    { "B_Mg",       "Mg",        "Magnesium",             "mg/l", "COMP", "2", "24.30",    "*"  },
    { "B_Ca",       "Ca",        "Calcium",               "mg/l", "COMP", "2", "40.08",    "*"  },
    { "B_Sr",       "Sr",        "Strontium",             "mg/l", "COMP", "2", "87.62",    ""  },
    { "B_Ba",       "Ba",        "Barium",                "mg/l", "COMP", "2", "137.33",   ""  },
    // ..  VIB
    { "B_Cr_II",    "CrII",      "Chromium (II)  chromous", "mg/l", "COMP", "2", "52.00",    ""  },
    { "B_Cr_III",   "CrIII",     "Chromium (III) chromic",  "mg/l", "COMP", "3", "52.00",    ""  },
    // ..  VIIB
    { "B_Mn_II",    "MnII",      "Manganese (II)  manganous", "mg/l", "COMP", "2", "54.94", ""  },
    { "B_Mn_III",   "MnIII",     "Manganese (III) manganic",  "mg/l", "COMP", "4", "54.94",    ""  },
    // ..  VIIIB
    { "B_Fe_II",    "FeII",      "Iron (II)    ferrous",   "mg/l", "COMP", "2", "55.85",    ""  },
    { "B_Fe_III",   "FeIII",     "Iron (III)   ferric",    "mg/l", "COMP", "3", "55.85",    ""  },
    { "B_Co_II",    "CoII",      "Cobalt (II)  cobaltous", "mg/l", "COMP", "2", "58.93",    ""  },
    { "B_Co_III",   "CoIII",     "Cobalt (III) cobaltic",  "mg/l", "COMP", "3", "58.93",    ""  },
    { "B_Ni_II",    "NiII",      "Nickel (II)  nickelous", "mg/l", "COMP", "2", "58.69",    ""  },
    { "B_Ni_III",   "NiIII",     "Nickel (III) nickelic",  "mg/l", "COMP", "3", "176.07",   ""  },
    // ..  IB
    { "B_Cu_I",     "Cu",        "Copper (I) cuprous",    "mg/l", "COMP", "1", "63.55",    ""  },
    { "B_Cu_II",    "CuII",      "Copper (II) cupric",    "mg/l", "COMP", "2", "63.55",    ""  },
    { "B_Ag",       "Ag",        "Silver",                "mg/l", "COMP", "1", "107.87",   ""  },
    { "B_Au",       "Au",        "Gold aurous",           "mg/l", "COMP", "1", "196.97",   ""  },
    { "B_Au_III",   "AuIII",     "Gold (III) auric",      "mg/l", "COMP", "1", "196.97",   ""  },
    // ..  IIB
    { "B_Zn",       "Zn",        "Zinc",                  "mg/l", "COMP", "2", "65.39",    ""  },
    { "B_Cd",       "Cd",        "Cadmium",               "mg/l", "COMP", "2", "112.41",   ""  },
    { "B_Hg_I",     "Hg",        "Mercury (I) mercurous", "mg/l", "COMP", "2", "200.59",   ""  },
    { "B_Hg_II",    "HgII",      "Mercury (II) mercuric", "mg/l", "COMP", "2", "200.59",   ""  },
    // ..  IIIA
    { "B_Al",       "Al",        "Aluminum",              "mg/l", "COMP", "3", "26.98",    ""  },
    //.. VA
    { "B_SB_III",   "SbIII",     "Antimony (III) antimonous", "mg/l", "COMP", "3", "121.75",   ""  },
    { "B_SB_V",     "SbV",       "Antimony (V)   antimonic",  "mg/l", "COMP", "5", "121.75",   ""  },
    { "B_BI_III",   "BiIII",     "Bismuth (III)  bismuthous", "mg/l", "COMP", "3", "208.98",   ""  },
    { "B_BI_V",     "BiV",       "Bismuth (V)    bismuthic",  "mg/l", "COMP", "5", "208.98",   ""  },
    // ..  IVA
    { "B_Sn_II",    "SnII",      "Tin (II)  stannous",    "mg/l", "COMP", "2", "118.71",   ""  },
    { "B_Sn_IV",    "SnIV",      "Tin (IV)  stannic",     "mg/l", "COMP", "4", "118.71",   ""  },
    { "B_Pb_II",    "PbII",      "Lead (II) plumbous",    "mg/l", "COMP", "2", "207.2",    ""  },
    { "B_Pb_IV",    "PbIV",      "Lead (IV) plumbic",     "mg/l", "COMP", "4", "207.2",    ""  },
    // .. Other
    { "B_NH4",      "NH4",       "Ammonium",              "mg/l", "COMP", "1", "18.04",    ""  },

    // Common Anions

    // ..Halogen
    { "B_F",        "F",         "Floride",               "mg/l", "COMP", "1", "19.0",     ""  },
    { "B_Cl",       "Cl",        "Chloride",              "mg/l", "COMP", "1", "35.45",    "*"  },
    { "B_Br",       "Br",        "Bromide",               "mg/l", "COMP", "1", "79.90",    "*"  },
    { "B_I",        "I",         "Iodine",                "mg/l", "COMP", "1", "126.9",    "*"  },

    { "B_OH",       "OH",        "Hydroxide",             "mg/l", "COMP", "1", "17.0",     ""  },

    { "B_BO3",      "BO3",       "Borate",                "mg/l", "COMP", "3", "58.81",    ""  },
    { "B_CO3",      "CO3",       "Carbonate",             "mg/l", "COMP", "2", "60.01",    "*"  },
    { "B_HCO3",     "HCO3",      "Bicarbonate",           "mg/l", "COMP", "1", "61.02",    "*"  },

    { "B_ClO",      "ClO",       "Hypochlorite",          "mg/l", "COMP", "1", "51.45",    ""  },
    { "B_ClO2",     "ClO2",      "Chlorite",              "mg/l", "COMP", "1", "67.45",    ""  },
    { "B_ClO3",     "ClO3",      "Chlorate",              "mg/l", "COMP", "1", "83.45",    ""  },
    { "B_ClO4",     "ClO4",      "Per chlorate",          "mg/l", "COMP", "1", "99.45",    ""  },

    { "B_CN",       "CN",        "Cyanide",               "mg/l", "COMP", "1", "26.02",    ""  },
    { "B_NCO",      "NCO",       "Cyanate",               "mg/l", "COMP", "1", "42.02",    ""  },
    { "B_OCN",      "OCN",       "Isocyanate",            "mg/l", "COMP", "1", "42.02",    ""  },
    { "B_SCN",      "SCN",       "Thicyanite",            "mg/l", "COMP", "1", "58.09",    ""  },

    { "B_N",        "N",         "Nitride",               "mg/l", "COMP", "3", "14.01",    ""  },
    { "B_N3",       "N3",        "Azide",                 "mg/l", "COMP", "3", "42.03",    ""  },
    { "B_NO2",      "NO2",       "Nitrite",               "mg/l", "COMP", "1", "46.01",    ""  },
    { "B_NO3",      "NO3",       "Nitrate",               "mg/l", "COMP", "1", "62.01",    ""  },

    { "B_CrO4",     "CrO4",      "Chromate",              "mg/l", "COMP", "2", "116.0",    ""  },
    { "B_Cr2O7",    "Cr2O7",     "Dichromate",            "mg/l", "COMP", "2", "216.0",    ""  },

    { "B_MnO4",     "MnO4",      "Permanganate",          "mg/l", "COMP", "1", "118.94",   ""  },

    { "B_P",        "P",         "Phosphide",             "mg/l", "COMP", "3", "30.97",    ""  },
    { "B_PO3",      "PO3",       "Phosphite",             "mg/l", "COMP", "3", "94.97",    ""  },
    { "B_PO4",      "PO4",       "Phosphate",             "mg/l", "COMP", "3", "94.97",    ""  },
    { "B_HPO4",     "HPO4",      "Monohydrogen Phosphate","mg/l", "COMP", "2", "95.97",    ""  },
    { "B_H2PO4",    "H2PO4",     "Dihydrogen Phosphate",  "mg/l", "COMP", "1", "96.97",    ""  },

    { "B_As",       "As",        "Arsinide",              "mg/l", "COMP", "2", "74.92",    ""  },
    { "B_Se",       "Se",        "Selenide",              "mg/l", "COMP", "2", "78.96",    ""  },

    { "B_S",        "S",         "Sulfide",               "mg/l", "COMP", "2", "32.07",    ""  },
    { "B_HS",       "HS",        "Hydrogen Sulfide",      "mg/l", "COMP", "1", "33.07",    ""  },
    { "B_SO3",      "SO3",       "Sulphite",              "mg/l", "COMP", "2", "80.07",    ""  },
    { "B_HSO3",     "HSO3",      "Hydrogen Sulphite",     "mg/l", "COMP", "1", "81.07",    ""  },
    { "B_S2O3",     "S2O3",      "Thiosulphate",          "mg/l", "COMP", "2", "112.14",   ""  },
    { "B_SO4",      "SO4",       "Sulfate",               "mg/l", "COMP", "2", "96.06",    "*"  },
    { "B_HSO4",     "HSO4",      "Bisulfate",             "mg/l", "COMP", "1", "97.06",    ""  },

    // -- Totals
    { "B_SOLID",    "SOLID",     "Total Solids",          "mg/l", "DATA",    "",    "",     ""  },
    { "B_TDS",      "TDS",       "Total Solids Computed", "mg/l", "DATA",    "",    "",     ""  },

    // -- Well ID
    { "B_NAME",     "NAME",      "Well Name",             "NONE", "LOC",     "",    "",     ""  },
    { "B_API",      "API",       "API-Number",            "NONE", "LOC",     "",    "",     ""  },
    // -- Other Info
    {  "B_FIELD",   "FIELD",     "Field",                 "NONE", "LOC",     "",    "",     ""  },
    // -- Location
    {  "B_COUNTY",  "COUNTY",    "County",                "NONE", "LOC",     "",    "",     ""  },
    {  "B_STATE",   "STATE",     "State",                 "NONE", "LOC",     "",    "",     ""  },
    {  "B_LOC",     "LOC",       "Location",              "NONE", "LOC",     "",    "",     ""  },
    {  "B_LAT",     "LAT",       "Latitude",              "deg",  "LOC",     "",    "",     ""  },
    {  "B_LONG",    "LONG",      "Longitude",             "deg",  "LOC",     "",    "",     ""  },
    // -- Depth Information
    { "B_KB",       "KB",        "Kelly Bushing",         "ft",   "ELEV",    "",    "",     ""  },
    { "B_GL",       "GL",        "Ground Level",          "ft",   "ELEV",    "",    "",     ""  },
    { "B_DF",       "DF",        "Derrick Floor",         "ft",   "ELEV",    "",    "",     ""  },
    { "B_TGT",      "TGT",       "TGT",                   "ft",   "ELEV",    "",    "",     ""  },
    // -- Source Information
    { "B_SRC",      "SRC",       "source",                "NONE", "SAMPLE",  "",    "",     ""  },
    { "B_DATE",     "DATE",      "Recovery date",         "NONE", "SAMPLE",  "",    "",     ""  },
  };

  /** Method getValence()
   *  <p> This method will retrieve the String value of Valence from
   *      the CURVES Array and convert to integer.
   */

  public static final int getValence( int iValue )
  {
    int iValence = 0;

    if ((iValue >= _LI) && (iValue < _SOLID))
    {
	  iValence = cmn.cmnString.stringToInt( CURVES[iValue][5] );
	  iValence = Math.abs(iValence);
	}

    return (iValence);
  }

  /** Method getGMperMole()
   *  <p> This method will retrieve the String value of gm/mole from
   *      the CURVES Array and convert to double.
   */

  public static final double getGMperMole( int iValue )
  {
    double dValue = 0.0;

    if ((iValue >= _LI) && (iValue < _SOLID))
    {
	  dValue = cmn.cmnString.stringToDouble( CURVES[iValue][6] );
	}

    return (dValue);
  }
}