/*
 * @dstParameters.java Version 1.1 03/20/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package dst;

/** Class dstParameters
 *  <p> This Class will provide parse constants for the LAS 3 Read Class
 *
 *  @version 1.1 03/20/2012
 *  @author  John R. Victorine
 */

public class dstParameters
{
  public static final int _ID    = 0;
  public static final int _FORM  = 1;
  public static final int _STRT  = 2;
  public static final int _STOP  = 3;
  public static final int _DEV   = 4;
  public static final int _WHIP  = 5;
  public static final int _SERL  = 6;
  public static final int _READ  = 7;
  public static final int _PRES  = 8;
  public static final int _DEPT  = 9;
  public static final int _CMMNT = 10;
  public static final int _HOLE  = 11;
  public static final int _COND  = 12;

  public static final int _CAP   = 13;
  public static final int _STIM  = 14;
  public static final int _ETIM  = 15;
  public static final int _TIMC  = 16;
  public static final int _TIMON = 17;
  public static final int _TIMOF = 18;
  public static final int _TYPE  = 19;
  public static final int _UNIT  = 20;
  public static final int _BASE  = 21;
  public static final int _ENG   = 22;

  public static final int _MUD   = 23;
  public static final int _MUDD  = 24;
  public static final int _MUDV  = 25;
  public static final int _FL    = 26;
  public static final int _RM    = 27;
  public static final int _MSAL  = 28;
  public static final int _CAKE  = 29;
  public static final int _CSHT  = 30;
  public static final int _CSHL  = 31;
  public static final int _CSHV  = 32;
  public static final int _GCSHT = 33;
  public static final int _GCSHP = 34;
  public static final int _OAPI  = 35;
  public static final int _WSAL  = 36;

  // -- Quantitative Analysis Panel

  public static final int _m     = 37;   // Slope
  public static final int _q     = 38;   // Flow Rate
  public static final int _h     = 39;   // Thickness
  public static final int _u     = 40;   // Viscosity
  public static final int _Pr    = 41;   // Formation Pressure
  public static final int _degR  = 42;   // Formation Temperature
  public static final int _beta  = 43;   // Formation volume Factor      (FLUID)
  public static final int _Z     = 44;   // Super compressibility Factor (GAS)

  public static final int _KHU   = 45;   // Transmissibility       (COMPUTED)
  public static final int _KH    = 46;   // Permeability Thickness (COMPUTED)
  public static final int _K     = 47;   // Effective Permeability (COMPUTED)

  public static final int _Pif   = 48;   // Initial Static Reservoir Pressure
  public static final int _Pwf   = 49;   // Bottom Hole Flowing Pressure (FFP)
  public static final int _tf    = 50;   // Flow Time
  public static final int _rw    = 51;   // Well Bore radius
  public static final int _re    = 52;   // Drainage radius
  public static final int _PHI   = 53;   // Porosity
  public static final int _c     = 54;   // Compressibility
  public static final int _G     = 55;   // Specific Gravity of Gas      (GAS)

  public static final int _Pwfc  = 56;   // Bottom Hole Flowing Pressure (COMPUTED) (GAS)

  public static final int _DR    = 57;   // Damage Ratio
  public static final int _S     = 58;   // Skin Factor
  public static final int _Ps    = 59;   // Pressure Drop due to Skin Factor
  public static final int _Pt    = 60;   // Pressure Drop due to Turbulence  (GAS)

  public static final int _PI    = 61;   // Productive Index
  public static final int _RI    = 62;   // Radius of Investigation

  public static final int _qa    = 63;   // Settled Production Rate (FLUID)
  public static final int _qAOF  = 64;   // Absolute Open Flow      (GAS)

  public static final int _ELP1  = 65;   // Exclude Points Data Points Below Shut In 1
  public static final int _EUP1  = 66;   // Exclude Points Data Points Above Shut In 1

  public static final int _M1    = 67;   // Shut In 1 Slope
  public static final int _P1    = 68;   // Shut In 1 Intercept

  public static final int _ELP2  = 69;   // Exclude Points Data Points Below Shut In 1
  public static final int _EUP2  = 70;   // Exclude Points Data Points Above Shut In 1

  public static final int _M2    = 71;   // Shut In 1 Slope
  public static final int _P2    = 72;   // Shut In 1 Intercept

  public static final int _RECTL = 73;
  public static final int _RECTV = 74;
  public static final int _RECC  = 75;

  public static final int _TOTAL = 76;

  public static String PARAMETERS[][] = {
    { "ID   ", "",      "",       "DST ID",                         "S", "" },
    { "FORM ", "",      "",       "Formation",                      "S", "" },
    { "STRT ", "FT",    "",       "Interval Depth From",            "F", "" },
    { "STOP ", "FT",    "",       "Interval Depth To",              "F", "" },

    { "DEV  ", "",      "",       "Deviated",                       "S", "" },
    { "WHIP ", "FT",    "",       "Wipstock",                       "F", "" },
    { "SERL ", "",      "",       "Serial Number",                  "S", "" },
    { "READ ", "",      "",       "Reading (Inside/Outside)",       "S", "" },
    { "PRES ", "PSI",   "",       "Pressure",                       "F", "" },
    { "DEPT ", "FT",    "",       "At Depth",                       "F", "" },
    { "CMMNT", "",      "",       "Test Comments",                  "S", "" },
    { "HOLE ", "IN",    "",       "Hole Diameter",                  "F", "" },
    { "COND ", "",      "",       "Hole Condition",                 "S", "" },

    { "CAP  ", "PSI",   "",       "Capacity",                       "F", "" },
    { "STIM ", "DATE",  "",       "Start Date/Time",                "MM/DD/YYYY-HHMMSS", "" },
    { "ETIM ", "DATE",  "",       "End Date/Time Logger",           "MM/DD/YYYY-HHMMSS", "" },
    { "TIMC ", "DATE",  "",       "Last Calibration",               "MM/DD/YYYY-HHMMSS", "" },
    { "TIMON", "DATE",  "",       "Time on Bottom",                 "MM/DD/YYYY-HHMMSS", "" },
    { "TIMOF", "DATE",  "",       "Time off Bottom",                "MM/DD/YYYY-HHMMSS", "" },
    { "TYPE ", "",      "",       "Test Type",                      "S", "" },
    { "UNIT ", "",      "",       "Logging Unit Number",            "S", "" },
    { "LGGR ", "",      "",       "DST Logging Company",            "S", "" },
    { "ENG  ", "",      "",       "Recording Engineer",             "S", "" },

    { "MUD  ", "",      "",       "Mud type",                       "S", "" },
    { "MUDD ", "GM/CC", "",       "Mud Density",                    "F", "" },
    { "MUDV ", "SEC/Qt","",       "Mud Viscosity (Funnel)",         "F", "" },
    { "FL   ", "CI",   "",        "Fluid Loss",                     "F", "" },
    { "RM   ", "OHM-M", "",       "Resistivity of Mud",             "F", "" },
    { "MSAL ", "PPM",   "",       "Mud Salinity",                   "F", "" },
    { "CAKE ", "IN",    "",       "Filter Cake",                    "F", "" },
    { "CSHT ", "",      "",       "Cusion Type",                    "S", "" },
    { "CSHL ", "FT",    "",       "Cusion Length",                  "F", "" },
    { "CSHV ", "BBL",   "",       "Cusion Volume",                  "F", "" },
    { "GCSHT", "",      "",       "GAS Cusion Type",                "S", "" },
    { "GCSHP", "PSI",   "",       "GAS Cusion Length",              "F", "" },
    { "OAPI ", "DEG API","",      "Oil API",                        "F", "" },
    { "WSAL ", "PPM",   "",       "Water Salinity",                 "F", "" },

    { "SLOPE", "PSI/CYCLE", "",   "Horner Plot Slope",              "F", "" },
    { "FLOW ", " STB/DAY ", "",   "Flow Rate",                      "F", ""  },
    { "THICK", "FT",        "",   "Vertical Thickness of Continuous Porosity", "F", ""  },
    { "VIS  ", "CP",        "",   "Viscosity",                      "F", ""  },
    { "FPRES", "PSI",       "",   "Formation Pressure",             "F", ""  },
    { "FTEMP", "DEG F",     "",   "Formation Temperature",          "F", ""  },
    { "BETA ", "RB/STB",    "",   "Formation Volume Factor",        "F", ""  },
    { "Z    ", "",          "",   "Gas Deviation Factor",           "F", ""  },

    { "KHU  ", "MD-FT/CP",  "",   "Transmissibility (Kh/u)",        "F", ""  },
    { "KH   ", "MD-FT",     "",   "Permeability Thickness (Kh)",    "F", ""  },
    { "PERM ", "MD",        "",   "Effective Permeability (K)",     "F", ""  },

    { "PIS  ", "PSI",       "",   "Initial Static Reservoir Pressure", "F", ""  },
    { "PWF  ", "PSI",       "",   "Bottom Hole Flowing Pressure",   "F", ""  },
    { "TIMEF", "MIN",       "",   "Flow Time",                      "F", ""  },
    { "RADIS", "FT",        "",   "Well Bore Radius",               "F", ""  },
    { "DRAIN", "FT",        "",   "Drainage Radius",                "F", ""  },
    { "PHI  ", "PU",        "",   "Porosity",                       "F", ""  },
    { "COMPS", "vol/vol/psi","",  "Compressibilities",              "F", ""  },
    { "GRAV ", "",          "",   "Specific Gravity of Gas (air=1)", "F", ""  },

    { "PWFC ", "PSI",       "",   "Flowing Pressure Corrected for Turbulence", "F", ""  },

    { "DR   ", "",          "",   "Damage Ratio",                    "F", ""  },
    { "S    ", "",          "",   "Skin Factor",                     "F", ""  },
    { "DELPS", "PSI",       "",   "Pressure Drop due to Skin",       "F", ""  },
    { "DELPT", "PSI",       "",   "Pressure Drop due to Turbulence", "F", ""  },

    { "PI   ", "STB/DAY/PSI","",  "Productive Index",                "F", ""  },
    { "RI   ", "FT",        "",   "Radius of Investigation",         "F", ""  },

    { "QS   ", "STB/DAY",   "",   "Settled Production Rate",         "F", ""  },
    { "AOF  ", "MCF/DAY",   "",   "Absolute Open Flow",              "F", ""  },

    { "ELP1 ", "",          "",   "Exclude Points Data Points Below Shut In 1", "F", "" },
    { "EUP1 ", "",          "",   "Exclude Points Data Points Above Shut In 1", "F", "" },

    { "M1   ", "PSI/CYCLE", "",   "Shut In 1 Slope",                 "F", "" },
    { "P1   ", "PSI",       "",   "Shut In 1 Intercept",             "F", ""  },

    { "ELP2 ", "",          "",   "Exclude Points Data Points Below Shut In 2", "F", "" },
    { "EUP2 ", "",          "",   "Exclude Points Data Points Above Shut In 2", "F", "" },

    { "M2   ", "PSI/CYCLE", "",   "Shut In 2 Slope",                 "F", "" },
    { "P2   ", "PSI",       "",   "Shut In 2 Intercept",             "F", ""  },

    { "RECTL", "FT",        "",   "Recovery Total Length",          "F", "" },
    { "RECTV", "BBL",       "",   "Recovery Total Volume",          "F", "" },
    { "RECC ", "",          "",   "Recovery Comments",              "S", "" },
  };

}
