/*
 * @rockColumnStruct.java Version 1.1 08/02/2012
 *
 * Copyright (c) 2012 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package lith.rock;

/** Class rockColumnStruct
 *  <p> This Class holds the Rock Column Depth Data.
 *
 *  @version 1.1 08/02/2012
 *  @author  John R. Victorine
 */

public class rockColumnStruct
{
  public static final int _NONE      = -1;
  public static final int _TRACE     = 0; // a visible mark (sparse)
  public static final int _SLIGHTLY  = 1; // a little (some - unspecified amount) (few)
  public static final int _NORMAL    = 2; // evident
  public static final int _MANY      = 3; // large number
  public static final int _VERY      = 4; // Extremely - exceedingly; abundant - great plenty
  public static final int _PRIMARY   = 5; // Primary Lithology

  public static final String AMOUNT[] = { "trace", "slightly", "", "many", "very", "primary" };

  public String sKEY          = "0";  // Unique Identifier

  public double depthStart    = 0.0;  // Starting Depth
  public double depthEnd      = 0.0;  // Ending Depth

  public int    iRGB[]        = { 255, 255, 255 }; // Background Color

  public String sPrimary      = "";   // Primary Lithology Identifier

  public int    iSecondary    = 0;    // Total Number of Secondary Lithology
  public int    iAmount[]     = null; // Amount of Secondary Lithology using above identifiers.
  public String secondary[]   = null; // Array of Secondary Lithology ID's

  /** Method delete()
   * <p> This method will set the arrays to null in order to get Java to
   *     deallocate memory for the object.
   */

  public void delete()
  {
    sKEY        = null;
    iRGB        = null;
    iAmount     = null;
    secondary   = null;
  }
}

/*
 *  @version 1.1 08/02/2012
 *  @author  John Victorine
 */
