/*
 * @quakeEventFrame.java Version 1.1 05/19/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import iqstrat.iqstratStruct;        // Global IQSTRAT Data Structure
import iqstrat.iqstratHeadersStruct; // Well Header Data Structure

import quake.gui.quakeEventPanel;    // Zonation Table Panel

import event.eventFileDataStruct;    // Zonation File Data Structure

/** Class quakeEventFrame
 *  <p> This Class will allow the user to select the status of the data
 *
 *  @version 1.1 05/19/2015
 *  @author  John R. Victorine
 */

public class quakeEventFrame extends JFrame implements ActionListener
{
  // Input Variables

  private Observable           notifier = null; // Observables Object
  private iqstratStruct        stStruct = null; // Global IQSTRAT Data Structure
  private iqstratHeadersStruct stHeader = null; // Header Data Structure
  private eventFileDataStruct  stData   = null; // Zonation File Data Structure

  // Global Frame Variables

  private quakeEventPanel      panel    = null; // Zonation Table Panel

  // Global Frame Widgets

  private JButton btnClose   = null; // Close Dialog

  // Well Information Plot Data

  private JLabel lblName   = new JLabel(); // Well Name
  private JLabel lblAPI    = new JLabel(); // API-Number
  private JLabel lblStatus = new JLabel(); // Well Status
  private JLabel lblLat    = new JLabel(); // Latitude
  private JLabel lblLong   = new JLabel(); // Longitude
  private JLabel lblTD     = new JLabel(); // Total Depth
  private JLabel lblElev   = new JLabel(); // Elevation

  /** Constructor quakeEventFrame()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = IQSTRAT Data Structure
   *  @param stData    = Zonation Data Structure
   */

  public quakeEventFrame( iqstratStruct        stStruct,
                          iqstratHeadersStruct stHeader,
                          eventFileDataStruct  stData )
  {
    try
    {
      this.stStruct = stStruct;
      this.stHeader = stHeader;
      this.stData   = stData;

      jbInit();

      setWellHeaderLabels();

      addWindowListener( new quakeEventFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop      = new JPanel();
    JPanel pnlBTN      = new JPanel();
    JPanel pnl         = new JPanel();
    JPanel pnlHeader   = new JPanel();

    JButton btnI       = new JButton();
    JButton btnII      = new JButton();
    JButton btnIII     = new JButton();

    // Create Frame

    this.setTitle("Zonation Data from LAS Log Curves");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // Buttons Panel

    pnlBTN.setBorder(BorderFactory.createEtchedBorder());

    // -- File Menu Bar

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(27, 27));
    btnClose.setToolTipText("Close Data");
    btnClose.addActionListener(this);

    // .. Well Header Information Panel

    pnl.setLayout(new BorderLayout());
    pnlHeader = buildHeaderPanel();

    // Center Panel

    panel  = new quakeEventPanel(stStruct, stData);

    // Attach Widgets to frame

    this.getContentPane().add(pnlTop, BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                BorderLayout.WEST);
    pnlBTN.add(btnClose,              null);

    pnlTop.add(pnl,                   BorderLayout.CENTER);
    pnl.add(pnlHeader,                BorderLayout.WEST);

    this.getContentPane().add(panel,  BorderLayout.CENTER);

    // Display the Frame

//    this.setSize(new Dimension(400, 650));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setSize(new Dimension(d.width-5, d.height-50));
    this.setSize(new Dimension(600, d.height-50));
    this.setLocation(d.width-600,1);
//    this.setLocation((d.width - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** Method buildHeaderPanel()
   *  <p> This method will build the Summary Well Header Information Panel
   *  @return panel = the Summary Well Header Information Panel
   */

  private JPanel buildHeaderPanel()
  {
	JPanel panel           = new JPanel();

    JPanel pnlInfo         = new JPanel();
    JPanel pnlInfoPosition = new JPanel();
    JPanel pnlInfoXY       = new JPanel();
    JPanel pnlInfoZ        = new JPanel();

    JPanel pnl1            = new JPanel();
    JPanel pnl2            = new JPanel();
    JPanel pnl3            = new JPanel();

    JPanel pnlID           = new JPanel();
    JPanel pnlName         = new JPanel();
    JPanel pnlAPI          = new JPanel();
    JPanel pnlStatus       = new JPanel();

    TitledBorder titledBorderWell = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Header Information:");
    titledBorderWell.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Header Information Plot Labels, & Cross Section Depth Range Panels

    panel.setLayout(new BorderLayout());
    panel.setBorder(titledBorderWell);
//    panel.setPreferredSize(new Dimension(800, 27));

    // ---- Header Information Plot Labels Panel

    pnlInfo.setLayout(new BorderLayout());

    pnl1.setLayout(new BorderLayout());
    pnl2.setLayout(new BorderLayout());
    pnl3.setLayout(new BorderLayout());

    // .. Well Information Panel

    pnlID.setLayout(new BorderLayout());

    // .... Well Name Panel

    pnlName.setLayout(new BorderLayout());

    lblName.setText("Name: ");
//    lblName.setSize(200, 27);

    // .... Well API-Number & Well Status Panel

    pnlAPI.setLayout(new BorderLayout());

    lblAPI.setText(" API-Number: ");

    pnlStatus.setLayout(new BorderLayout());

    lblStatus.setText(" Status:");

    // .. Well Position Panel

    pnlInfoPosition.setLayout(new GridLayout());

    // .... XY Well Position Panel

    pnlInfoXY.setLayout(new GridLayout());

    lblLat.setText(" Latitude: ");
    lblLong.setText("Longitude: ");

    // .... Z Well Position Panel

    pnlInfoZ.setLayout(new GridLayout());

    lblTD.setText("Depth: ");
    lblElev.setText("Elev (GL): ");

    // Attach Widgets to Panel

    panel.add(pnlInfo,             BorderLayout.WEST);

    pnlInfo.add(pnlName,           BorderLayout.CENTER);
    pnlName.add(lblName,           BorderLayout.CENTER);

    pnlInfo.add(pnlAPI,            BorderLayout.EAST);
    pnlAPI.add(lblAPI,             BorderLayout.CENTER);

    panel.add(pnl1,                BorderLayout.CENTER);
    pnl1.add(pnlInfoPosition,      BorderLayout.WEST);

    pnlInfoPosition.add(pnlInfoXY, null);
    pnlInfoXY.add(lblLat,          null);
    pnlInfoXY.add(lblLong,         null);

    pnlInfoPosition.add(pnlInfoZ,  null);
    pnlInfoZ.add(lblTD,            null);
    pnlInfoZ.add(lblElev,          null);


    pnl1.add(pnl2,                 BorderLayout.CENTER);
    pnl2.add(pnlStatus,            BorderLayout.WEST);
    pnlStatus.add(lblStatus,       BorderLayout.CENTER);


	return (panel);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** METHOD quakeEventFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quakeEventFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e) { close(); }
  }

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
    // Input Variables

    stStruct     = null; // Global IQSTRAT Data Structure
    stHeader     = null; // Header Information Data
    stData       = null; // Zonation Data Structure

    panel        = null;

    btnClose     = null; // Close Dialog

    // Well Information Plot Data

    lblName      = null; // Well Name
    lblAPI       = null; // API-Number
    lblStatus    = null; // Well Status
    lblLat       = null; // Latitude
    lblLong      = null; // Longitude
    lblTD        = null; // Total Depth
    lblElev      = null; // Elevation

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setWellHeaderLabels()
   *  <p> This method will set the Well Header Label Fields
   */

  private void setWellHeaderLabels()
  {
    if (stHeader != null)
    {
      lblName.setText( "Name: "      + stHeader.sName );
      lblAPI.setText( " API-Number: " + stHeader.sAPI  + " ");
      lblStatus.setText( " Status: "  + stHeader.status );
      lblLat.setText( " Latitude: "   + stHeader.dLatitude );
      lblLong.setText( " Longitude: " + stHeader.dLongitude );
      lblTD.setText( " Total Depth: " + stHeader.depth );

      lblElev.setText(" Elevation: ");
      if (stHeader.dGL > 0.0)
        lblElev.setText("Elevation (GL): " + stHeader.dGL);
      else if (stHeader.dKB > 0.0)
        lblElev.setText("Elevation (KB): " + stHeader.dKB);
      else if (stHeader.dDF > 0.0)
        lblElev.setText("Elevation (DF): " + stHeader.dDF);
    }
    else
    {
      lblName.setText( "Name: " );
      lblAPI.setText( " API-Number: " );
      lblStatus.setText( " Status: " );
      lblLat.setText( " Latitude: " );
      lblLong.setText( " Longitude: " );
      lblTD.setText( " Total Depth: " );

      lblElev.setText(" Elevation: ");
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnClose)   { close(); }
  }
}


/*
 *  @version 1.1 05/19/2015
 *  @author  John Victorine
 */
