/*
 * @quakeEventTable.java Version 1.1 05/19/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.Observable;
import java.math.BigDecimal;

import gui.guiTable;

import iqstrat.iqstratStruct;      // Global IQSTRAT Data Structure
import event.eventFileDataStruct;  // Zonation File Data Structure

/** Class quakeEventTable
 *  <p> This Class will display the Zonation Data.
 *
 *  @version 1.1 05/19/2015
 *  @author  John R. Victorine
 */

public class quakeEventTable
{
  public static final int _TOP      = event.eventStandardTools._TOP;
  public static final int _BASE     = event.eventStandardTools._BASE;

  public static final int _LITH     = event.eventStandardTools._LITH;

  public static final int _VAc      = event.eventStandardTools._VAc;
  public static final int _VAs      = event.eventStandardTools._VAs;

  public static final int _TSP      = event.eventStandardTools._TSP;

  public static final int TOTAL_LAS = event.eventStandardTools.TOTAL_LAS;
  public static final int TOTAL     = event.eventStandardTools.TOTAL;

  private int    iRows        = 0;
  private int    iColumns     = 9;
  private String sColumn[]    = { "By Log [ft]", "By Elev [ft]",
                                  "Name",     "Lithology",
                                  "",
                                  "Delta SP [msec]",
                                  "",
                                  "Avg Vp [ft/s]", "Avg Vs [ft/s]"
                                };

  private int    iColLength[] = { 12, 12,
                                  20, 20,
                                   2,
                                  13,
                                   2,
                                  10, 10
                                };
/*
  private boolean  bColumn[]  = { false, false,
                                  true,  true,
                                  false,
                                  false,
                                  false,
                                  false, false
                                };
*/
  private Object              oData[][] = null;
  private guiTable            pTable    = null;

  private Observable          notifier  = null; // Observable Object
  private eventFileDataStruct st        = null; // Zonation Data Structure
  private int                 iChange   = 0;

  private iqstratStruct       stStruct  = null; // Global IQSTRAT Data Structure

  /** Constructor dstPressureSummaryTable()
   *  <p> This is the Constructor for this class.
   */

  public quakeEventTable(iqstratStruct stStruct)
  {
	this.stStruct = stStruct;

//    pTable = new guiTable(bColumn, 0, iColumns, sColumn);
    pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Constructor quakeEventTable()
   *  <p> This is the Constructor for this class.
   */

  public quakeEventTable(iqstratStruct stStruct, eventFileDataStruct st)
  {
	this.stStruct = stStruct;
	this.st       = st;

    populateList();
/*
    if (iRows > 0)
      pTable = new guiTable(bColumn, 0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(bColumn, 0, iColumns, sColumn);
*/
    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn    = null;
    iColLength = null;
    oData      = null;
    pTable     = null;

    notifier   = null; // Observable Object
    st         = null;
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iColLength[i]);
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method populateList()
   *  <p> This method will populate the Zonation data.
   */

  private void populateList()
  {
    int        i,j;
    int        iCount  = 0;
    double     data    = 0.0;
    String     sLith   = "";
    BigDecimal bigData = new BigDecimal( "0.0" );

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (st != null)
    {
      iChange = 1;
      iCount  = 0;
      oData   = new Object[st.iRows][iColumns];

      for (i=0; i<st.iRows; i++)
      {
		oData[i][0] = new String(""+st.getRow(_TOP, i));

		if (st.dElev > 0.0)
		  data      = st.dElev - st.getRow(_TOP, i);
		else
		  data      = st.getRow(_TOP, i);
		oData[i][1] = new String(""+data);

   	    oData[i][2] = new String(st.sName[i]);

//		if (st.sName[i].length() == 0)
//		  oData[i][2] = new String(st.sTops[i]);

		data        = st.getRow(_LITH, i);

        for (int k=0; k<stStruct.stSymbols.iCount; k++)
        {
		  if ((int)data == stStruct.stSymbols.stItem[k].id)
		  {
            sLith = new String(stStruct.stSymbols.stItem[k].sName);
		  }
		}

        if (data != -1.0)
          oData[i][3] = new String(""+sLith);
        else
          oData[i][3] = new String("");

        oData[i][4] = new String("");

		oData[i][5] = new String("");

        data = st.getRow(_TSP, i);
        if (data != st.dNull)
		  oData[i][5] = new String(""+data);

        oData[i][6] = new String("");

		oData[i][7] = new String("");
		oData[i][8] = new String("");

        data = st.getRow(_VAc, i);
        if (data != st.dNull)
		  oData[i][7] = new String(""+data);
        data = st.getRow(_VAs, i);
        if (data != st.dNull)
		  oData[i][8] = new String(""+data);

        iCount++;
	  }
    }

    iRows = iCount;
  }

  /** Method setData()
   *  <p> This method will reload the data.
   * @param st = The Zonation Data Structure
   */

  public void setData(eventFileDataStruct st)
  {
    this.st = st;

    populateList();

    pTable.reload(iRows, oData);

    if (iChange == 0)
      iRows = 0;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 05/19/2015
 *  @author  John Victorine
 */
