/*
 * @quakeWellPanel.java Version 1.1 05/12/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.util.Observable;
import java.util.Observer;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

import javax.swing.*;
import javax.swing.event.*;

import cmn.cmnStruct;                    // Common Data Structure

import iqstrat.iqstratStruct;            // Global IQSTRAT Data Structure
import iqstrat.iqstratHeadersListStruct; // Headers Data Structure
import iqstrat.iqstratHeadersStruct;     // Headers Data Structure

import las.lasFileDataStruct;            // LAS File Data Structure

import quake.gui.quakeWellListTable;     // Well Data List Table

import event.gui.eventMapCSVFrame;       // Import Tops Data CSV File
import event.eventFileDataStruct;        // Zonation File Data Structure
import quake.gui.quakeEventFrame;        // Display Zonation Time/velocity data

import event.eventXDataListStruct;       // Event X List Structure
import event.eventXDataStruct;           // Event X Data Structure

/** Class quakeWellPanel
 *  <p> This Class will create the Zonation Well Data Panel
 *
 *  @version 1.1 05/12/2015
 *  @author  John R. Victorine
 */

public class quakeWellPanel extends JPanel implements ActionListener, Observer
{
  // Input Variables

  private iqstratStruct stStruct  = null; // IQSTRAT Data Structure
  private cmnStruct     stCMN     = null; // Common Data Structure
  private Observable    pNotifier = null; // Observables Object

  // Global Variables

  private Observable notifier  = null; // Observables Object

  private quakeWellListTable pnlTable = null; // Well List Table
  private eventMapCSVFrame   pZeke    = null; // Read Zonation Dialog
  private quakeEventFrame    pEvent   = null; // Display Zonation Dialog

  // Data Source Location

  public static final int _PC         = las.lasPlotLoad._PC;

  // -- Type of data

  public static final int _HEADERS    = 0;  // Headers Data

  // -- LAS File & Geological Data

  public static final int _ZEKE       = 31; // Zonation Data

  // .. Header Information Data Structure

  private iqstratHeadersListStruct stHeaders = null; // Header List Structure
  private iqstratHeadersStruct     stHeader  = null; // Header Data Structure

  // .. LAS Data

  private eventFileDataStruct      stZeke    = null; // Zonation Data Structure
  private lasFileDataStruct        stLASZeke = null; // LAS File Data Struct

  // .. Multiple "Well" Data

  private eventXDataListStruct     stList    = null; // Well Data List Structure

  // Global Frame Widgets

  private JButton btnZeke   = null;  // Zonation Button
  private JButton btnSelect = new JButton();
  private JButton btnDelete = new JButton();

  /** Constructor quakeWellPanel()
   *  <p> This is the Constructor for this class.
   *  @param stStruct  = IQSTRAT Data Structure
   *  @param pNotifier = Observable
   */

  public quakeWellPanel( iqstratStruct stStruct, Observable pNotifier )
  {
    try
    {
      this.stStruct  = stStruct;
      this.stCMN     = stStruct.stCMN;
      this.pNotifier = pNotifier;

      jbInit();
      setButtons();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel      pnlIcon   = new JPanel();
	JPanel      pnlCenter = new JPanel();
	JPanel      pnlbtns   = new JPanel();

    JScrollPane scroll    = new JScrollPane();

    TitledBorder titledBorderTitle = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(168, 168, 168)),
        "Sonic Average Time Difference Data by Bed from Well Data");
    titledBorderTitle.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderZEKE = new TitledBorder(
        BorderFactory.createEmptyBorder(), "Beds CSV");
    titledBorderZEKE.setTitleJustification(TitledBorder.CENTER);
    titledBorderZEKE.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Build Observable

    notifier = new quakeWellPanelObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    this.setLayout(new BorderLayout());
    this.setPreferredSize( new Dimension(10, 150));
    this.setBorder( titledBorderTitle );

    // Zonation ICON Panel

    pnlIcon.setBackground(Color.white);
    pnlIcon.setBorder( titledBorderZEKE );

    btnZeke = new JButton( createImageIcon( cmn.cmnStruct.ZEKE ) );
    btnZeke.setBackground(Color.white);
    btnZeke.setPreferredSize( new Dimension(64, 64) );
    btnZeke.addActionListener( this );

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

    // -- Well List Table Panel

    pnlTable = new quakeWellListTable();
    scroll  = pnlTable.getScrollPane();

    // -- Buttons Panel

    pnlbtns.setLayout(new GridLayout());

    btnSelect.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSelect.setBorderPainted(true);
    btnSelect.setText("View Time & Velocity Data");
    btnSelect.addActionListener(this);

    btnDelete.setFont(new java.awt.Font("Dialog", 1, 11));
    btnDelete.setBorderPainted(true);
    btnDelete.setText("Delete Well Data");
    btnDelete.addActionListener(this);

    // Attach Widgets to Panel

    this.add( pnlIcon,       BorderLayout.WEST );
    pnlIcon.add( btnZeke,    null);

    this.add(pnlCenter,      BorderLayout.CENTER );
    pnlCenter.add( scroll,   BorderLayout.CENTER );
    pnlCenter.add( pnlbtns,  BorderLayout.SOUTH );
    pnlbtns.add( btnSelect, null );
    pnlbtns.add( btnDelete, null );
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    pNotifier  = null;
    stCMN      = null; // Common Data Structure

    // Global Variables

    notifier   = null; // Top Observables Object

    if (pnlTable != null)
      pnlTable.close();
    pnlTable  = null;

    if (pZeke != null)
      pZeke.close();
    pZeke    = null; // Zonation Dialog

    if (pEvent != null)
      pEvent.close();
    pEvent   = null; // Display Zonation Dialog

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;      // Header Information Data

    // LAS Data

    if (stZeke != null)
      stZeke.delete();
    stZeke = null;

    if (stLASZeke != null)
      stLASZeke.delete();
    stLASZeke = null;

    // Global Frame Widgets

    btnZeke   = null;  // Zonation Button
    btnDelete = null;  // Delete from list
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   * <p> This method will retrieve data
   * @param iLocation = Location of Data being retrieved
   * @param iData     = Data Type being retrieved
   */

  private void getData( int iLocation, int iData)
  {
    switch (iData)
    {
      case _HEADERS:    // Headers Data
        stHeader = iqstrat.iqstratHeadersUtility.copy( pZeke.getHeaderData() );
        break;

      case _ZEKE:
        stZeke    = event.eventFileDataUtility.transfer( pZeke.getData() );
  	    stLASZeke = event.eventMapToLAS.mapZekeToLAS( stZeke );
        stLASZeke = las.lasFileDataUtility.computeMath( stLASZeke );

        pZeke.close();
        break;
    }
  }

  /** Method getEventData()
   *  <p> This method will retrieve the Event X Data Structure
   *  @param st = Event X Data Structure
   */

  public eventXDataStruct getEventData()
  {
    eventXDataStruct     st  = null;
    iqstratHeadersStruct stH = null;

    if (pnlTable != null)
    {
	  if (pnlTable.getTotalRows() > 0)
	  {
	    stH = pnlTable.getRowData();

	    if (stList != null)
	    {
		  for (int i=0; i<stList.iCount; i++)
		  {
//System.out.println(stList.stItem[i].sKEY+" <> "+stH.sKEY);
		    if (stList.stItem[i].sKEY.equals(stH.sKEY))
		    {
		  	  st = stList.stItem[i];
		    }
		  }
		}
	  }
	}

    return (st);
  }

  /** Method getTotalRows()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getTotalRows() { return (pnlTable.getTotalRows()); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable or disable buttons depending on the data.
   */

  private void setButtons()
  {
	btnSelect.setEnabled(false);
	btnDelete.setEnabled(false);

	if (pnlTable != null)
	{
	  if (pnlTable.getTotalRows() > 0)
	  {
     	btnSelect.setEnabled(true);
   	    btnDelete.setEnabled(true);
	  }
	}

    if (pNotifier != null)
      pNotifier.notifyObservers( new String("Get Well Table Rows") );
  }

  /** Method setData()
   *  <p> This method will build the event X Data Structure
   */

  private eventXDataStruct setData()
  {
	eventXDataStruct st = new eventXDataStruct();

	st.stHeader = iqstrat.iqstratHeadersUtility.copy( stHeader );
	st.stZeke   = event.eventFileDataUtility.transfer( stZeke );
	st.stLAS    = las.lasFileDataUtility.transfer( stLASZeke );

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

    if (stZeke != null)
      stZeke.delete();
    stZeke = null;

    if (stLASZeke != null)
      stLASZeke.delete();
    stLASZeke = null;

    setButtons();
	return (st);
  }


  /** Method delete()
   *  <p> This method will delete a row from the table
   */

  private void delete()
  {
	iqstratHeadersStruct st = null;

	if (pnlTable != null)
	{
	  st =  pnlTable.getRowData();

	  stList = event.eventXDataUtility.remove( st.sKEY, stList );
	  stHeaders = iqstrat.iqstratHeadersUtility.remove( stHeaders, st.sKEY );
      pnlTable.setData( stHeaders );
	}
  }

  /* ===================================================================== *
   * ------------------------- SEARCH DIALOGS ---------------------------- *
   * ===================================================================== */

  /** Method search_ZEKE()
   * <p> This method will search for a Zonation File that is on the user's PC
   */

  private void search_ZEKE()
  {
    FileDialog dialog      = null;
    Frame      frameTemp   = new Frame();
    String     sReadMethod = null;
    String     sDirectory  = null;
    String     sFilename   = null;

    dialog = new FileDialog(frameTemp,
             "Select Zonation Comma Delimited File from your PC.");
    dialog.setVisible(true);
    frameTemp.setVisible(true);

    iqstratStruct stStruct = new iqstratStruct();
    stStruct.stCMN         = stCMN;

    // Retrieve LAS File Location

    if ((dialog.getDirectory() != null) && (dialog.getFile() != null))
    {
      sReadMethod = new String("FILE");
      sDirectory  = new String(dialog.getDirectory());
      sFilename   = new String(dialog.getFile());

      if (pZeke != null)
        pZeke.close();
      pZeke = null;

      pZeke = new eventMapCSVFrame( notifier, sReadMethod, sDirectory, sFilename );
      pZeke.setStruct(stStruct);
    }

    // Dispose of Temporary Dialog & Frame

    frameTemp.dispose();
    frameTemp = null;

    dialog.dispose();
    dialog = null;
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnZeke)  { search_ZEKE(); }

    if (event.getSource() == btnSelect)
    {
      if (pEvent != null)
        pEvent.close();
      pEvent   = null; // Display Zonation Dialog

      eventXDataStruct st = getEventData();

      if (st != null)
      {
        pEvent = new quakeEventFrame( stStruct, st.stHeader, st.stZeke );
	  }
	}

    if (event.getSource() == btnDelete) { delete(); }

    setButtons();
  }

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String((String) arg);

    /* ------------------------------------------------------------------ *
     * --------------------- Zonation Data Actions ---------------------- *
     * ------------------------------------------------------------------ */

    if (sArg.equals("Zonation CSV File Loaded"))
    {
	  getData( _PC,  _HEADERS );
      getData( _PC,  _ZEKE );

      eventXDataStruct st = setData();

      if (!event.eventXDataUtility.exists( st, stList ))
      {
		st        = event.eventXDataUtility.setKEY( st );
        stHeaders = iqstrat.iqstratHeadersUtility.add( stHeaders, st.stHeader );
        pnlTable.setData( stHeaders );

        if (st.stHeader.dGL > 0.0)
          st.stZeke.dElev = st.stHeader.dGL;
        else if (st.stHeader.dKB > 0.0)
          st.stZeke.dElev = st.stHeader.dKB;
        else if (st.stHeader.dDF > 0.0)
          st.stZeke.dElev = st.stHeader.dDF;

        stList = event.eventXDataUtility.add( st, stList );
	  }

	  st.delete();
	  st = null;
	}

	setButtons();
  }
}

/** CLASS quakeWellPanelObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class quakeWellPanelObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 05/12/2015
 *  @author  John Victorine
 */
