/*
 * @WriteSQLFile.java Version 1.1 06/16/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.io;

import java.io.*;
import java.awt.*;

import util.utilFileIO;
import cmn.cmnString;

import quake.quakeSensorsDataListStruct; // Compute Quake Location List Structure
import quake.quakeSensorsStruct;         // Sensor Data Structure

/** Class WriteSQLFile
 *  <p> This class will parse the List Structure and
 *      write the information to the CSV File.
 *
 *  @version 1.1 06/16/2015
 *  @author  John R. Victorine
 */

public class WriteSQLFile
{

  /** Method write()
   *  <p> This is the method will write the CSV String to the user's PC
   *  @param sPath     = The Directory Path for the SQL File
   *  @param sFilename = The Filename of the SQL File.
   *  @param SQL       = The SQL String
   */

  public static void write(String sPath, String sFilename, String SQL)
  {
    utilFileIO pIO  = new utilFileIO();

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    // Write the SQL Block to the File

    pIO.Write( SQL );

    // Close the File

    pIO.Close();
  }

  /** Method buildSQL()
   *  <p> This method will build the SQL File
   *  @param  st  = quake location list data structure
   *  @param  stS = seismic sensor location data structure
   *  @return SQL = ASCII SQL String
   */

  public static String buildSQL( quakeSensorsDataListStruct st, quakeSensorsStruct stS )
  {
	int i,j;
	String SQL      = "";
	String sQuery   = "";
	int    iQuery[] = { 0, 0, 0, 0, 0 };
    String sKID     = "0";

    if (st != null)
    {
	  for (i=0; i<st.iCount; i++)
	  {
		for (j=0; j<5; j++)
		{
		  iQuery[j] = 0;
		}

		if (st.stItem[i].sTime.length() > 0)
		  iQuery[0] = 1;

		if (st.stItem[i].dMagnitude > 0.0)
		  iQuery[1] = 1;

		if (st.stItem[i].dLatitude  != 0.0)
		  iQuery[2] = 1;

		if (st.stItem[i].dLongitude != 0.0)
		  iQuery[3] = 1;

		sKID   = kgs.io.kgsIOKID.getKID();

        sQuery = new String("INSERT INTO co2_event_location\n ( KID, ");

        if (iQuery[0] > 0)
          sQuery = new String(sQuery + "event_time, ");

        if (iQuery[1] > 0)
          sQuery = new String(sQuery + "event_magnitude, ");

        if (iQuery[2] > 0)
          sQuery = new String(sQuery + "nad27_latitude, ");

        if (iQuery[3] > 0)
          sQuery = new String(sQuery + "nad27_longitude, ");

        sQuery = new String(sQuery + "depth, depth_units) VALUES \n (" + sKID + ", ");

        if (iQuery[0] > 0)
        {
          sQuery = new String(sQuery +
            "TO_DATE('" + st.stItem[i].sTime + "', 'MM/DD/YYYY HH24:MI'), " );
		}

        if (iQuery[1] > 0)
          sQuery = new String(sQuery + st.stItem[i].dMagnitude + ", " );

        if (iQuery[2] > 0)
          sQuery = new String(sQuery + st.stItem[i].dLatitude + ", " );

        if (iQuery[3] > 0)
          sQuery = new String(sQuery + st.stItem[i].dLongitude + ", " );

        sQuery = new String(sQuery + st.stItem[i].depth + ", 'ft');\nCOMMIT;\n\n");

        SQL = SQL + sQuery;
	  }
	}

    SQL = buildComptedDataSQL(SQL, st, stS);

	return (SQL);
  }

  /** Method buildComptedDataSQL()
   *  <p> This method will build the SQL File
   *  @param  SQL = ASCII SQL String
   *  @param  st  = quake location list data structure
   *  @param  stS = seismic sensor location data structure
   *  @return SQL = ASCII SQL String
   */

  public static String buildComptedDataSQL( String SQL,
	                                        quakeSensorsDataListStruct st,
                                            quakeSensorsStruct stS )
  {
	int i,j, k;
	String sQuery   = "";
	//                  C  Et Em Pw Vp Vs R
	int    iQuery[] = { 0, 0, 0, 0, 0, 0, 0 };
    String sKID     = "0";
    String sKID_S   = "0";

	if ((st!= null) && (stS != null))
	{
  	  for (i=0; i<st.iCount; i++)
	  {
		for (j=0; j<7; j++)
		{
		  iQuery[j] = 0;
		}

        if (st.stItem[i].sChannel.length() > 0)
          iQuery[0] =1;

		if (st.stItem[i].sTime.length() > 0)
		  iQuery[1] = 1;

		if (st.stItem[i].dMagnitude > 0.0)
		  iQuery[2] = 1;

	    for (k=0; k<3; k++)
	    {
		  for (j=3; j<7; j++)
		  {
		    iQuery[j] = 0;
		  }

          if (st.stItem[i].data[k][1] > 0)
            iQuery[3] = 1;

          if (st.stItem[i].data[k][2] > 0)
            iQuery[4] = 1;

          if (st.stItem[i].data[k][3] > 0)
            iQuery[5] = 1;

          if (st.stItem[i].data[k][4] > 0)
            iQuery[6] = 1;

	      for (j=0; j<stS.iTotal; j++)
	      {
	        if (stS.id[j] == (int) st.stItem[i].data[k][0])
	        {
		      sKID   = kgs.io.kgsIOKID.getKID();
              sKID_S = new String(stS.sKID[j]);
			}
		  }

          sQuery = new String("INSERT INTO co2_sensor_raw\n ( KID, sensor_KID,\n ");

          if (iQuery[0] > 0)
            sQuery = new String(sQuery + "channel, \n");

          if (iQuery[1] > 0)
            sQuery = new String(sQuery + "event_time,\n ");

          if (iQuery[2] > 0)
            sQuery = new String(sQuery + "event_magnitude,\n ");

          sQuery = new String(sQuery + "sensor_order,\n ");

          if (iQuery[3] > 0)
            sQuery = new String(sQuery + "sp_time_diff, sp_time_units,\n ");

          if (iQuery[4] > 0)
            sQuery = new String(sQuery + "Vp, Vs, velocity_units,\n ");

          if (iQuery[6] > 0)
            sQuery = new String(sQuery + "distance, distance_units\n ");

          sQuery = new String(sQuery + ") VALUES \n (" + sKID + ", " + sKID_S + ", \n");

          if (iQuery[0] > 0)
            sQuery = new String(sQuery + "'" + st.stItem[i].sChannel+ "', \n");

          if (iQuery[1] > 0)
            sQuery = new String(sQuery +
              "TO_DATE('" + st.stItem[i].sTime + "', 'MM/DD/YYYY HH24:MI'),\n " );

          if (iQuery[2] > 0)
            sQuery = new String(sQuery + st.stItem[i].dMagnitude + ", \n" );

          sQuery = new String(sQuery + k +", \n");

          if (iQuery[3] > 0)
            sQuery = new String(sQuery + st.stItem[i].data[k][1] +", 'msec', \n");

          if (iQuery[4] > 0)
            sQuery = new String(sQuery + st.stItem[i].data[k][2] + ", " +
                                       + st.stItem[i].data[k][3] + ", 'm/sec',\n ");

          if (iQuery[6] > 0)
            sQuery = new String(sQuery + st.stItem[i].data[k][4] +", 'm'\n ");

          sQuery = new String(sQuery + ");\nCOMMIT;\n\n");

  		  SQL = SQL + sQuery;
		}
	  }
	}

	return (SQL);
  }
}

/*
 *  @version 1.1 06/16/2015
 *  @author  John Victorine
 */
