/*
 * @quakeMath.java Version 1.1 02/09/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.math;

import java.lang.*;
import java.math.BigDecimal;

/** Class quakeMath
 *  <p> This Class will perform the math to compute depth of earthquake and
 *      its latitude and longitude.
 *
 *  @version 1.1 02/09/2015
 *  @author  John R. Victorine
 */

public class quakeMath
{
  public static double Re      = 6371.0;           // Earth Radius [km]
  public static double ReM1    = 0.00015696123;    // Inverse Earth Radius 1/[km]
  public static double RADIANS = 0.017453292;      // PI/180.0
  public static double ANTI_RADIANS = 57.29578049; // 180.0/PI
  public static double INV_THOUSAND = 0.001;
  public static double INV_THREE    = 0.33333333333;

  /** Method compute()
   *  This method will compute the depth, latitude and longitude of earthquake
   *  @param phi   = array of latitude sensors
   *  @param lamda = array of longitude sensors
   *  @param r     = array of sensors distance to earthquake
   *  @param dOut  = array of output earthquake position [latitude, longitude, depth]
   */

  public static double[] compute(double phi[], double lamda[],
                                 double r[], double h[])
  {
	int i = 0;
	double dOut[] = { 0.0, 0.0, 0.0 };

	double p[]    = { 0.0, 0.0, 0.0 };
	double l[]    = { 0.0, 0.0, 0.0 };
	double km[]   = { 0.0, 0.0, 0.0 };
	double hkm[]  = { 0.0, 0.0, 0.0 };

	double pavg   = 0.0;
	double CosAvg = 0.0;
	double ho     = 0.0;

	double x21    = 0.0;
	double y21    = 0.0;
	double x31    = 0.0;
	double y31    = 0.0;

	double xq1    = 0.0;
	double yq1    = 0.0;

	double D      = 0.0;
	double R1     = 0.0;
	double R2     = 0.0;
	double tmp    = 0.0;

	double PHIq   = 0.0;
	double LAMDAq = 0.0;
/*
    BigDecimal TEMP = new BigDecimal( "2.0000000000000000" );
    TEMP = math.mathBigDecimal.sqrt(TEMP);
//System.out.println("SQRT(2)="+TEMP.doubleValue()+" "+Math.sqrt(2.0));

    BigDecimal Rearth = new BigDecimal(new String(""+Re));
    BigDecimal RAD    = new BigDecimal(new String(""+RADIANS));
    BigDecimal DEG    = new BigDecimal(new String(""+ANTI_RADIANS));
    BigDecimal MICRO  = new BigDecimal(new String(""+INV_THOUSAND));
    BigDecimal THIRD  = new BigDecimal(new String(""+INV_THREE));
    BigDecimal TWO    = new BigDecimal(new String(""+2.0));
    BigDecimal ONE    = new BigDecimal( "1.0" );

    BigDecimal P[]    = new BigDecimal[3];
    BigDecimal L[]    = new BigDecimal[3];
    BigDecimal KM[]   = new BigDecimal[3];
    BigDecimal HKM[]  = new BigDecimal[3];

    BigDecimal PAVG   = new BigDecimal( "0.0" );
    BigDecimal CosAVG = new BigDecimal( "0.0" );
    BigDecimal H0     = new BigDecimal( "0.0" );

    BigDecimal X21    = new BigDecimal( "0.0" );
    BigDecimal Y21    = new BigDecimal( "0.0" );
    BigDecimal X31    = new BigDecimal( "0.0" );
    BigDecimal Y31    = new BigDecimal( "0.0" );

    BigDecimal Pnm    = new BigDecimal( "0.0" );
    BigDecimal Lnm    = new BigDecimal( "0.0" );

    BigDecimal TMP1   = new BigDecimal( "0.0" );
    BigDecimal TMP2   = new BigDecimal( "0.0" );
    BigDecimal TMP3   = new BigDecimal( "0.0" );
    BigDecimal TMP4   = new BigDecimal( "0.0" );

    BigDecimal Do     = new BigDecimal( "0.0" );
    BigDecimal A1     = new BigDecimal( "0.0" );
    BigDecimal A2     = new BigDecimal( "0.0" );

    BigDecimal XQ1    = new BigDecimal( "0.0" );
    BigDecimal YQ1    = new BigDecimal( "0.0" );
*/
	for (i=0; i<3; i++)
	{
//-      P[i]   = RAD.multiply(new BigDecimal(new String(""+phi[i])));
//-      L[i]   = RAD.multiply(new BigDecimal(new String(""+lamda[i])));
//-      KM[i]  = MICRO.multiply(new BigDecimal(new String(""+r[i])));
//-      HKM[i] = MICRO.multiply(new BigDecimal(new String(""+h[i])));
	  p[i]   = Math.PI * phi[i] / 180.0;
	  l[i]   = Math.PI * lamda[i] / 180.0;
	  km[i]  = r[i]/1000.0;
	  hkm[i] = h[i]/1000.0;
      tmp    = km[i]*km[i];
//System.out.println(i+" lat="+p[i]+" long="+l[i]+" r="+km[i]+" h="+hkm[i]+" r^2="+tmp);

//-      PAVG  = PAVG.add(P[i]);
//-      H0    = H0.add(HKM[i]);
	  pavg  = pavg + p[i];
	  ho    = ho + hkm[i];
	}

//-    PAVG    = PAVG.multiply(THIRD);
//-    H0      = H0.multiply(THIRD);
//-    CosAVG  = math.mathBigDecimal.cos(PAVG);
	ho      = ho/3.0;
	pavg    = pavg/3.0;
	CosAvg  = Math.cos(pavg);

//System.out.println("ho="+ho+" pavg="+pavg);

//-    Pnm     = P[1].subtract(P[0]);
//-    X21     = Rearth.multiply(math.mathBigDecimal.sin(Pnm));

//-    Lnm     = L[1].subtract(L[0]);
//-    Y21     = Rearth.multiply(math.mathBigDecimal.sin(Lnm));
//-    Y21     = Y21.multiply(CosAVG);

	x21     = Re * Math.sin(p[1]-p[0]);
	y21     = (Re * Math.sin(l[1]-l[0])) * Math.cos(pavg);

//System.out.println("x21="+x21+" y21="+y21);

//-    Pnm     = P[2].subtract(P[0]);
//-    X31     = Rearth.multiply(math.mathBigDecimal.sin(Pnm));

//-    Lnm     = L[2].subtract(L[0]);
//-    Y31     = Rearth.multiply(math.mathBigDecimal.sin(Lnm));
//-    Y31     = Y31.multiply(CosAVG);

	x31     = Re * Math.sin(p[2]-p[0]);
	y31     = (Re * Math.sin(l[2]-l[0])) * Math.cos(pavg);

//System.out.println("x31="+x31+" y31="+y31);
/*
    TMP1    = X31.multiply(Y21);
    TMP2    = X21.multiply(Y31);
    Do      = TMP1.subtract(TMP2);
    Do      = Do.multiply(TWO);

    TMP1    = X21.multiply(X21);
    TMP2    = Y21.multiply(Y21);
    TMP3    = KM[1].multiply(KM[1]);
    TMP4    = KM[0].multiply(KM[0]);
    A1      = TMP1.add(TMP2);
    A1      = A1.subtract(TMP3);
    A1      = A1.add(TMP4);

    TMP1    = X31.multiply(X31);
    TMP2    = Y31.multiply(Y31);
    TMP3    = KM[2].multiply(KM[2]);
    TMP4    = KM[0].multiply(KM[0]);
    A2      = TMP1.add(TMP2);
    A2      = A2.subtract(TMP3);
    A2      = A2.add(TMP4);
*/
	D       = 2.0 * ( x31*y21 - x21*y31 );
	R1      = x21*x21 + y21*y21 - km[1]*km[1] + km[0]*km[0];
	R2      = x31*x31 + y31*y31 - km[2]*km[2] + km[0]*km[0];

//System.out.println("D="+D+" R1="+R1+" R2="+R2);
/*
    TMP1    = A2.multiply(Y21);
    TMP2    = A1.multiply(Y31);
    XQ1     = TMP1.subtract(TMP2);
    XQ1     = XQ1.divide(Do, 6, BigDecimal.ROUND_UP);

    TMP2    = A2.multiply(X21);
    TMP1    = A1.multiply(X31);
    YQ1     = TMP1.subtract(TMP2);
    YQ1     = YQ1.divide(Do, 6, BigDecimal.ROUND_UP);
*/
	xq1     = (R2*y21 - R1*y31)/D;
	yq1     = (R2*x21 - R1*x31)/D;

//    tmp     = Math.sqrt(xq1*xq1 + yq1*yq1);
//System.out.println("q1:	xq1="+xq1+" yq1="+yq1+" sqrt(xq1*xq1+yq1*yq1)="+tmp);

   tmp     = km[0]*km[0] - xq1*xq1 - yq1*yq1;
//System.out.println("(Zo-H)^2="+tmp);

//-    TMP1    = KM[0].multiply(KM[0]);
//-    TMP2    = XQ1.multiply(XQ1);
//-    TMP3    = YQ1.multiply(YQ1);
//-    TMP4    = TMP1.subtract(TMP2);
//-    TMP4    = TMP4.subtract(TMP3);
//-    if (TMP4.doubleValue() > 0.0)
//-      TMP1  = math.mathBigDecimal.sqrt(TMP4);
//-    else
//-      TMP1  = new BigDecimal( "0.0" );
//-    TMP1    = HKM[0].subtract(TMP1);
//    TMP1    = TMP1.divide(ONE, 6, BigDecimal.ROUND_UP);
//-    TMP1    = TMP1.divide(MICRO, 6, BigDecimal.ROUND_UP);
//-    dOut[2] = TMP1.doubleValue();
    if (tmp > 0.0)
      dOut[2] = hkm[0] - Math.sqrt(km[0]*km[0] - xq1*xq1 - yq1*yq1);
    else
      dOut[2] = hkm[0];

    dOut[2] = dOut[2] * 1000.0;

//-    TMP1    = XQ1.divide(Rearth, 9, BigDecimal.ROUND_UP);
//-    TMP2    = P[0].add(math.mathBigDecimal.asin(TMP1));
//-    TMP2    = TMP2.multiply(DEG);
//-    TMP2    = TMP2.divide(ONE, 7, BigDecimal.ROUND_UP);
//-    dOut[0] = TMP2.doubleValue();

//-    TMP3    = Rearth.multiply(CosAVG);
//-    TMP1    = YQ1.divide(TMP3, 9, BigDecimal.ROUND_UP);
//-    TMP2    = L[0].add(math.mathBigDecimal.asin(TMP1));
//-    TMP2    = TMP2.multiply(DEG);
//-    TMP2    = TMP2.divide(ONE, 7, BigDecimal.ROUND_UP);
//-    dOut[1] = TMP2.doubleValue();

    PHIq    = p[0] + Math.asin(xq1/Re);
	LAMDAq  = l[0] + Math.asin(yq1/(Re*CosAvg));

	dOut[0] = 180.0 * PHIq / Math.PI;
	dOut[1] = 180.0 * LAMDAq / Math.PI;

    for (i=0; i<3; i++)
    {
	  dOut[i] = (double) ((int) (dOut[i] * 100000.0)) / 100000.0;
	}


//System.out.println("Latitude="+dOut[0]+" Longitude="+dOut[1]+" Depth = "+dOut[2]);

    return (dOut);
  }

  /** Method computeDistance()
   *  <p> This method will compute the distance from the Time Difference
   *      between the P and S Wave, millisec units.
   *  @param  Dt = The Time Difference between the P and S Wave
   *  @param  Vp = Average P-Wave Velocity
   *  @param  Vs = Average S-Wave Velocity
   *  @return R  = Distance from seismic event
   */

  public static double computeDistance( double Dt, double Vp, double Vs )
  {
	double R = 0.0;

    BigDecimal ONE  = new BigDecimal( "1.0" );
    BigDecimal BR   = new BigDecimal( "0.0" );
    BigDecimal BDt  = new BigDecimal(new String(""+Dt));
    BigDecimal BVp  = new BigDecimal(new String(""+Vp));
    BigDecimal BVs  = new BigDecimal(new String(""+Vs));
    BigDecimal TMP1 = new BigDecimal( "0.0" );
    BigDecimal TMP2 = new BigDecimal( "0.0" );
    BigDecimal TMP  = new BigDecimal( "0.0" );
    BigDecimal THSD = new BigDecimal( "0.001" );

	TMP1 = BVp.subtract(BVs);
	TMP2 = BVp.multiply(BVs);

	TMP  = TMP2.divide(TMP1, 4, BigDecimal.ROUND_UP);

	TMP1 = BDt.multiply(THSD);
	BR   = TMP1.multiply(TMP);
    BR   = BR.divide(ONE, 3, BigDecimal.ROUND_UP);

	R    = BR.doubleValue();

	return (R);
  }
}


/*
 *  @version 1.1 02/09/2015
 *  @author  John Victorine
 */
